/*
	Ninja Library
	
	COPYRIGHT (C) SEGA ENTERSRISES,LTD.
*/

#ifndef _NINJA_STR2_H_
#define _NINJA_STR2_H_

/*--------------------------------------*/
/*      Motion Structure                */
/*--------------------------------------*/
/*
 * NJS_MOTION_MDATA_INFO
 */
typedef struct
{
	void		*mdata;					/* NJS_MDATAx structure */
	void		**key;					/* Motion's pointer (mdata->p) */
	Uint32		*nbkeys;				/* array of the number of keyframesimdata->nbj */
	Float		frame;					/* current frame */
	Uint32		nbframes;				/* total number of keyframes */
	Uint32		cnt;					/* array pointer for NJS_MDATAx's members */
	NJS_MATRIX	*mtrx;					/* current matrix pointer  */
} NJS_MOTION_MDATA_INFO;

typedef	void	NJF_MOTION_OBJECT_FUNC( NJS_OBJECT *obj );
typedef	void	NJF_MOTION_MODEL_FUNC( NJS_MODEL *mdl );
typedef	void	NJF_MOTION_INT_F_FUNC( NJS_MKEY_F *key, Uint32 keyno, Uint32 n, Float *dst );
typedef	void	NJF_MOTION_INT_A_FUNC( NJS_MKEY_A *key, Uint32 keyno, Uint32 n, Angle *dst );
typedef	void	NJF_INT_F_FUNC( Float *src, Float *dst, Float u );
typedef	void	NJF_INT_A_FUNC( Angle *src, Angle *dst, Float u );
typedef	void	NJF_MOTION_ROTATE_FUNC( NJS_MATRIX *m, Angle x, Angle y, Angle z );


/*--------------------------------------*/
/*      Light Structure                 */
/*--------------------------------------*/

/*
 * NJS_LIGHT_QUEUE
 */
typedef struct {
	Int         ltptr[NJD_LIGHT_MAX];
	Int         num;            /* the maximum number of light  */
} NJS_LIGHT_QUEUE;

/*
 * NJS_LIGHT_CONTROLLER
 */
typedef struct {
	NJS_LIGHT_PTR   lt[NJD_LIGHT_MAX];
	Int             ltptr;      /* stack pointer                */
} NJS_LIGHT_CONTROLLER;

/*--------------------------------------*/
/*      Texture Structure               */
/*--------------------------------------*/
typedef struct tag_pvr_global_header
{
	Uint8   texHeaderChar[4];	/* 'GBIX' globalIndex TAG 	*/
	Uint32  nextPointer;		/* next pointer 			*/
	Uint32 	globalIndex;		/* globalIndex				*/
} NJS_PVR_GLOBAL_HEADER;

typedef struct tag_pvr_info_header
{
	Uint8   texHeaderChar[4];	/* 'PVRT' pvr TAG			*/
	Uint32  nextPointer;		/* next pointer				*/
	Uint32 	nTextureType;		/* texture attribute		*/
	Uint16	nWidth;				/* image size x				*/	
	Uint16	nHeight;			/* image size y				*/

} NJS_PVR_FILE_HEADER;

typedef struct tag_tga_file_header{
	Uint8 	Type;				/* type(0) 						*/
	Uint8	PaletteType;		/* palette type(0:RGB 1:Palette) */
	Uint8	ImageType;			/* image type(1,2,3,9,10,11,32,33) */
	Uint16	PalettePosition;	/* palette data position		*/
	Uint16	PaletteNum;			/* palette number				*/
	Uint8	PaletteBit;			/* palette bit counts			*/
	Uint8	XPosiyion;			/* image x position				*/
	Uint8	YPosition;			/* image y position 			*/
	Uint16	XSize;				/* image x size					*/
	Uint16	YSize;				/* image y size					*/
	Uint8	ColorBit;			/* color bit counts 8,16,24,32 	*/
	Uint8	AttrBit;			/* attribute bit	 			*/
} NJS_TGA_FILE_HEADER;

typedef struct tag_tga_palette{
	Uint8 b;					/* blue 						*/
	Uint8 g;					/* green 						*/
	Uint8 r;					/* red 							*/
} NJS_TGA_PALETTE;

typedef struct tag_bitmap_file_header{
	Uint8 	bfType[2];		/* BMP TAG ("BM") 					*/
	Uint32 	bfSize;			/* file size [byte] 				*/
	Uint16	bfReserved1;	/* reserved 1 [byte] (0)			*/
	Uint16	bfReserved2;	/* reserved 2 [byte] (0)			*/
	Uint32	bfOffBits;		/* header size [byte] 				*/
} NJS_BITMAP_FILE_HEADER;

typedef struct tag_bmp_info_header{
	Uint32 	biSize;			/* info header size [byte](40)		*/
	Sint32	biWidth;		/* image x size [dot] 				*/
	Sint32	biHeight;		/* image y size [dot] 				*/
	Uint16	biPlanes;		/* (1)								*/
	Uint16	biBitCount;		/* coloar bit number [bit/dot]		*/
	Uint32	biCompression;	/* compression type (0)				*/
	Uint32	biSizeImage;	/* compressed size 	(0)				*/
	Sint32	biXPelsPerMeter;/* x pels per meter (0)				*/
	Sint32	biYPelsPerMeter;/* y pels per meter (0)				*/
	Uint32	biClrUsed;		/* color number (0)					*/
	Uint32	biClrImportant;	/* color important(0)				*/
} NJS_BITMAP_INFO_HEADER;		

typedef struct tag_bmp_palette{
	Uint8 b;				/* blue */
	Uint8 g;				/* green */
	Uint8 r;				/* red */
	Uint8 a;				/* alpha */
} NJS_BITMAP_PALETTE;

typedef struct tag_loadtfunc{
	Sint32 (*getinfo)(NJS_TEXINFO* info,Uint32 attr);
	Sint32 (*getsurface)(NJS_TEXINFO* info,Uint32 attr);
	Sint32 (*loadtex)(NJS_TEXINFO* info,Uint32 attr);
}NJS_LOADTFUNC;

/*--------------------------------------*/
/*      Graphic2D Structure             */
/*--------------------------------------*/

typedef void (NJF_2D_DRAW_INITFUNC)(Float, Int);
typedef void (NJF_2D_DRAW_FUNC)(NJS_POINT2COL *, Int);

typedef	void (NJF_2D_DRAW_CIRCLE_INITFUNC)(NJS_POINT2COL *p2c, Int a, Int b);
typedef	void (NJF_2D_DRAW_COL_CIRCLE_DRAWFUNC)(KMVERTEX0 *a, KMVERTEX0 *b);
typedef	void (NJF_2D_DRAW_TEX_CIRCLE_DRAWFUNC)(KMVERTEX3 *a, KMVERTEX3 *b);
typedef void NJF_2D_DRAW_TEX_CIRCLE_LOOP( NJF_2D_DRAW_TEX_CIRCLE_DRAWFUNC *func );
typedef void NJF_2D_DRAW_COL_CIRCLE_LOOP( NJF_2D_DRAW_COL_CIRCLE_DRAWFUNC *func );

typedef struct {
	Float u;
	Float v;
	Float px;
	Float py;
	Float pu;
	Float pv;
	Int xcen;
	Int ycen;
	Int xrad;
	Int yrad;
	Int insflag;
	Int plyflag;
	Uint32 color;
} NJS_2D_DRAW_CIRCLE_ATTR;



#endif /* _NINJA_STR2_H_ */
