/*
    This program is sound utility program.
    Uses on Sound sample.
*/
# include "sndutls.h"
/*
    AttachSector

    Name: AttachSector
    Synopsis:
        AttachSector( Sint32 size )
    Description:
        Normaly GD File system read file unit of sector, but file size not always
        sector size boundary. AttachSector fix any size of data to sector boundary
        size. This function mostly uses on memory allcation size.
*/
Uint32 AttachSector( Sint32 size ) {
  Uint32 ret;
  ret = ( size + SECTOR_SIZE - 1 ) / SECTOR_SIZE;
  return ret * SECTOR_SIZE;
}

/*
    SoundInitialize

    Name: SoundInitialize
    Synopsis:
        SoundInitialize( const char *DrvName, const char *MltName )
    Description:
        SoundInitialize read sound driver and multi unit file from GD-ROM,
        and Initialize all sound sysytem such as hup driver and transfer data.
*/
void SoundInitialize( const char *DrvName, const char *MltName ) {
  char           *gSndDrvPtr;
  Sint32         gSndDrvSize;
  SDMEMBLK       mem_blk;
  GDFS           fd;
  char           *mlt;
  Sint32         mlt_size;
  Uint32         size;

  /* Load Sound Driver */
  if ( ( fd = gdFsOpen( DrvName, NULL ) ) != NULL ) {
    {
      gdFsGetFileSize( fd, &gSndDrvSize );
      size = AttachSector( gSndDrvSize );
      gSndDrvPtr = ( char * )syMalloc( size * sizeof( char ) );
      gdFsRead( fd, size / SECTOR_SIZE, gSndDrvPtr );
    }
    gdFsClose( fd );
  } else PrgStop(1);

  /*                                 */
  /* Initialize sound library module */
  /*                                 */
  sdLibInit( NULL, 0, 0 );
  sdMemBlkCreate( &mem_blk );
  {
    /* Initialize sound driver */
    sdMemBlkSetPrm( mem_blk, gSndDrvPtr, gSndDrvSize, SDD_MEMBLK_SYNC_FUNC, NULL );
    sdDrvInit( mem_blk);

    /*                      */
    /* Load Multi Unit File */
    /*                      */
    if ( ( fd = gdFsOpen( MltName, NULL ) ) != NULL ) {
      {
        gdFsGetFileSize( fd, &mlt_size );
        size = AttachSector( mlt_size );
        mlt = ( char * )syMalloc( size * sizeof( char ) );
        gdFsRead( fd, size / SECTOR_SIZE, mlt );
        sdMemBlkSetPrm( mem_blk, mlt, mlt_size, SDD_MEMBLK_SYNC_FUNC, NULL );
        sdDownloadMultiUnit( mem_blk );
      }
      gdFsClose( fd );
    } else PrgStop(2);
  }
  sdMemBlkDestroy( mem_blk );

  syFree( gSndDrvPtr );
  syFree( mlt );

}


void PrgStop( int n ) 
{
  while ( -1 );
}

