/**********************************************************************

	Title :                 Task module
	Filename :              task.c
	Author :                Thomas Szirtes
	Revision :              1.1

	Notes :                 General purpose simple multi-tasker.
							Linked list implementation
							Possible improvement: inline small functions

	Copyright :             (C) SEGA Europe 1998

  **********************************************************************/


/*
 *	Includes
 */

#include "ftypes.h"

#ifdef USING_SET4
 #include        "shinobi.h"
#else
 #include        "ninja.h"
 #include        "malloc.h"
#endif

#include        "task.h"





/*--------------------------------------------------------------------------

  TASK *TASK_Insert( TASK *t )

  Parameters:
		task to insert after
		
  Returns:
		new created task

  Notes:
	
  --------------------------------------------------------------------------*/
TASK *TASK_Insert( TASK *t )
{
	register TASK *nt;

	nt      =       TASK_New();

	/* put in list */
	nt->prev                =       t;
	nt->next                =       t->next;

	if(t->next!=NULL)
		(t->next)->prev		=       nt;
	t->next                 =       nt;

	return  nt;
}



/*--------------------------------------------------------------------------

  TASK *TASK_Delete( TASK *t )

  Parameters:
		t	-	point to task to delete

  Returns:
		pointer to the next task in list after deleted task		

  Notes:
	
  --------------------------------------------------------------------------*/
TASK *TASK_Delete( TASK *t )
{
	register        TASK *pt;

	if(t->data!=NULL) {
#ifdef USING_SET4
		syFree(t->data);
#else
		free(t->data);
#endif
	}

	/* remove from list */
	if(t->next!=NULL)
		(t->next)->prev =   t->prev;

	if(t->prev!=NULL)
		(t->prev)->next =       t->next;

	pt      =       t->next;

#ifdef USING_SET4
	syFree(t);
#else
	free(t);
#endif

	return  pt;
}


/*--------------------------------------------------------------------------

	TASK *TASK_New( void )

  Parameters:
 
  Returns:
		pointer to new created task

  Notes:
	
  --------------------------------------------------------------------------*/

TASK *TASK_New( void )
{
	register TASK   *nt;

#ifdef USING_SET4
	nt      =       syMalloc(sizeof(TASK));
#else
	nt      =       malloc(sizeof(TASK));
#endif

	nt->func		=		NULL;
	nt->prev        =       NULL;
	nt->next        =       NULL;
	nt->data        =       NULL;

	return nt;
}



/*--------------------------------------------------------------------------

  void TASK_Process( TASK *t )

  Parameters:
		t	-	task to process(execute)

  Returns:
		none

  Notes:
		the task gets aged - and if dies is removed from list
	
  --------------------------------------------------------------------------*/

void TASK_Process( TASK *t )
{
	
	if(t->func!=NULL)
		(t->func)(t);

	/* aging  */
	if (t->life != -1)
	{
		if( t->life == 0)
		{
			t = TASK_Delete(t);
		}
		else
		{
			t->life --;
		}
	}

	return;
}



/*--------------------------------------------------------------------------

  void TASK_ProcessAll(TASK *t )

  Parameters:
		pointer to top of task list to execute

  Returns:

  Notes:
		executes all tasks to end of list

  --------------------------------------------------------------------------*/
void TASK_ProcessAll(TASK *t )
{
	TASK    *nt;

	while (t != NULL)
	{
		nt              =       t->next;
		TASK_Process(t);
		t               =       nt;
	}
}


/*--------------------------------------------------------------------------

  void TASK_DeleteAll(TASK *t )

  Parameters:
		root task to delete

  Returns:

  Notes:
	
  --------------------------------------------------------------------------*/
void TASK_DeleteAll(TASK *t )
{
	while (t!=NULL)
	{
		t       =       TASK_Delete(t);
	}
}
