#--------------------------------------------------------------------------------------------#
#  CONFIDENTIAL AND PROPRIETARY:                                                             #
#       Copyright 1998 Sega of America.  All rights reserved.                                #
#       This sample source code is provided for demonstration purposes only.                 #
#       Use and distribution of Sega proprietary material is governed by                     #
#       developer licensing agreement.                                                       #
#--------------------------------------------------------------------------------------------#

#--------------------------------------------------------------------------------------------#
#  Name:     Katana.inc                                                                      #
#  Title:    Katana Makefile                                                                 #
#  Author:   Gary Lake                                                                       #
#  Created:  January 22, 1998                                                                #
#                                                                                            #
#  Version:  2.0                                                                             #
#  Platform: Set4 | Hitachi | Shinobi | Kamui | Ninja                                        #
#                                                                                            #
#  Description:                                                                              #
#       Shared Makefile definitions required to build Katana projects.                       #
#                                                                                            #
#  History:                                                                                  #
#       1/22/98 - Program created (Gary Lake).                                               #
#       4/28/98 - Modified for Set4 (Gary Lake).                                             #
#       5/15/98 - Updated for Set4 P2-J (Gary Lake).                                         #
#--------------------------------------------------------------------------------------------#

.SILENT :
.SUFFIXES : .c .cpp .src .bin .lib .elf


#--------------------------------------------------------------------------------------------#
# Directory path names                                                                       #
#--------------------------------------------------------------------------------------------#

# Katana SDK paths

LIBC_PATH       = ..\..\..\libc
KAMUI_PATH      = ..\..\..\kamui
NINJA_PATH      = ..\..\..\ninja
SHINOBI_PATH    = ..\..\..\shinobi

# Output files

OUTDIR          = .\exe
OBJDIR          = .\obj


#--------------------------------------------------------------------------------------------#
# Standard definitions                                                                       #
#--------------------------------------------------------------------------------------------#

# Compiler, Linker, and Assembler

C               = shc.exe
CPP             = shcpp.exe
ASM             = asmsh.exe
LINK            = lnk.exe
LIBR            = lbr.exe
DWFCONV         = dwfcnv.exe
ELFCONV         = cnvs.exe
BINCONV         = s2bin.exe

# Default libraries and object files

DEF_LIBS        = "$(SHINOBI_PATH)\lib\shinobi.lib"

DEF_OBJS        = "$(SHINOBI_PATH)\lib\strt1.obj"\
                  "$(SHINOBI_PATH)\lib\strt2.obj"


#--------------------------------------------------------------------------------------------#
# Compiler switches                                                                          #
#--------------------------------------------------------------------------------------------#

# Debugging on/off

!ifdef DEBUG
C_DEBUG         = -optimize=1 -debug
!else
C_DEBUG         = -optimize=1 -speed -nodebug
!endif

!ifdef LIST
C_LIST          = -listfile=$*.lst
!else
C_LIST          =
!endif


#--------------------------------------------------------------------------------------------#
# Compiler defines                                                                           #
#--------------------------------------------------------------------------------------------#

C_DEFS          = __SHC__,__SET4__,_STRICT_UNION_


#--------------------------------------------------------------------------------------------#
# Compiler options                                                                           #
#--------------------------------------------------------------------------------------------#

C_OPTS          = -cpu=sh4\
                  -division=cpu\
                  -endian=little\
                  -fpu=single\
                  -pic=0\
                  -macsave=0\
                  -string=const\
                  -comment=nonest\
                  -sjis\
                  -section=p=P,c=C,d=D,b=B\
                  -extra=a=400\
                  -show=obj,source,expansion,w=80,l=0\
                  -define=$(C_DEFS)\
                  -include="$(LIBC_PATH)\include","$(KAMUI_PATH)\include",\
                  "$(NINJA_PATH)\include","$(SHINOBI_PATH)\include",.

C_FLAGS         = -sub="$(OBJDIR)\shc.sub" $(C_DEBUG) $(C_LIST)


#--------------------------------------------------------------------------------------------#
# Assembler switches                                                                         #
#--------------------------------------------------------------------------------------------#

# Debugging on/off

!ifdef DEBUG
ASM_DEBUG       = -debug=d
!else
ASM_DEBUG       =
!endif

!ifdef LIST
ASM_LIST        = -list=$*.lst
!else
ASM_LIST        =
!endif


#--------------------------------------------------------------------------------------------#
# Assembler options                                                                          #
#--------------------------------------------------------------------------------------------#

ASM_OPTS        = -cpu=sh4\
                  -endian=little\
                  -sjis\
                  -private

ASM_FLAGS       = $(ASM_OPTS) $(ASM_DEBUG) $(ASM_LIST)


#--------------------------------------------------------------------------------------------#
# Linker switches                                                                            #
#--------------------------------------------------------------------------------------------#

# Debugging on/off

!ifdef DEBUG
LINK_DEBUG      = -debug
!else
LINK_DEBUG      =
!endif

!ifdef LIST
LINK_LIST       = -print=$*.map
!else
LINK_LIST       =
!endif


#--------------------------------------------------------------------------------------------#
# Linker options                                                                             #
#--------------------------------------------------------------------------------------------#

LINK_OPTS       = -align_section\
                  -elf\
                  -form=a\
                  -entry=_START_SYS\
                  -start=DSGLH(8C010000)

LINK_FLAGS      = $(DEF_OBJS) $(PROJECT_OBJS) -library=$(DEF_LIBS),$(PROJECT_LIBS)\
                  $(LINK_OPTS) $(LINK_DEBUG) $(LINK_LIST)


#--------------------------------------------------------------------------------------------#
# Build rules                                                                                #
#--------------------------------------------------------------------------------------------#

# Build C/C++ files

.c{$(OBJDIR)}.obj:
    echo $(C_OPTS) > "$(OBJDIR)\shc.sub"
    "$(C)" $< -ob=$@ $(C_FLAGS)
!ifdef DEBUG
    "$(DWFCONV)" -sysrof=$*.o $*.obj
    copy $*.o $*.obj
!endif

.cpp{$(OBJDIR)}.obj:
    echo $(C_OPTS) > "$(OBJDIR)\shc.sub"
    "$(CPP)" $< -ob=$@ $(C_FLAGS)
!ifdef DEBUG
    "$(DWFCONV)" -sysrof=$*.o $*.obj
    copy $*.o $*.obj
!endif

# Build ASM files

.src{$(OBJDIR)}.obj:
    "$(ASM)" $< -O=$@ $(ASM_FLAGS)

# Link object files into exe

"$(OUTDIR)\$(PROJECT).elf" : "$(OUTDIR)" "$(OBJDIR)" $(PROJECT_OBJS)
    "$(LINK)" $(LINK_FLAGS) -output=$@
!ifdef BIN
    "$(ELFCONV)" "$(OUTDIR)\$(PROJECT).elf"
    "$(BINCONV)" "$(OUTDIR)\$(PROJECT).mot" "$(OUTDIR)\$(PROJECT).bin"
!endif


#--------------------------------------------------------------------------------------------#
# Makefile directives                                                                        #
#--------------------------------------------------------------------------------------------#

# Clean project files

CLEAN :
    del "$(OUTDIR)\*.elf"
    del "$(OUTDIR)\*.map"
    del "$(OUTDIR)\*.bin"
    del "$(OUTDIR)\*.mot"
    del "$(OBJDIR)\*.dwf"
    del "$(OBJDIR)\*.lst"
    del "$(OBJDIR)\*.obj"
    del "$(OBJDIR)\*.o"
    del "$(OBJDIR)\*.sub"

# Create directories

"$(OUTDIR)" :
    if not exist "$(OUTDIR)/$(NULL)" mkdir "$(OUTDIR)"

"$(OBJDIR)" :
    if not exist "$(OBJDIR)/$(NULL)" mkdir "$(OBJDIR)"