/*
 *-----------------------------------------------------------------------------
 *  FILE: sg_bup.h
 *
 *      Copyright(c) 1998 SEGA System R&D Dept.
 *
 *  PURPOSE:
 *
 *      VMSpt@CVXewb_
 *
 *  DESCRIPTION:
 *
 *  AUTHOR(S):
 *
 *      1998/3/30   S.Uchida Ver.0.90
 *
 *-----------------------------------------------------------------------------
 */

#ifndef _SG_BUP_H_
#define _SG_BUP_H_

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

/*
 * C VIRTUAL TYPES DEFINITIONS
 */

#include <sg_xpt.h>


/******** ^CX^v\ *********************************************/
typedef struct {
	Uint16 year;                /* N(1998`)                              */
	Uint8 month;                /* (1`12)                               */
	Uint8 day;                  /* (1`31)                               */
	Uint8 hour;                 /* (0`23)                               */
	Uint8 minute;               /* (0`59)                               */
	Uint8 second;               /* b(0`59)                               */
	Uint8 dayofweek;            /* j( = 0, = 1, ... y = 6)         */
} BUS_TIME;


/******** fBXN\ ***********************************************/
typedef struct {
	Sint8 volume[36];           /* {[x                        */
	Uint16 total_blocks;        /* SubN                            */
	Uint16 total_user_blocks;   /* S[U[ubN(őt@C)    */
	Uint16 free_blocks;         /* 󂫃ubN                          */
	Uint16 free_user_blocks;    /* 󂫃[U[ubN                  */
	Uint16 block_size;          /* ubNTCY(512Œ)                 */
	Uint16 icon_no;             /* ACRԍ(0`255)                    */
	BUS_TIME time;              /* tH[}bgꂽ                  */
} BUS_DISKINFO;

/******** fBXN\ ***********************************************/
typedef struct {
	Uint32 filesize;            /* t@CTCY(oCg)                */
	Uint16 blocks;              /* gpubN                          */
	Uint8 type;                 /* t@C^Cv(ʏ/s\)           */
	Uint8 copyflag;             /* Rs[tO(ff=Rs[s)             */
	Uint16 headerofs;           /* wb_ItZbg                        */
	BUS_TIME time;              /* ^CX^v                          */
} BUS_FILEINFO;


/******** t@Cnh(o̒ڎQƋ֎~) ***************************/
typedef struct {
	Uint8 isOpen;
	Uint8 copyflag;
	Uint16 hdrofs;
	Sint32 open_mode;
	Sint32 type;
	Sint32 open_num;
	char fname[16];
	Uint8 date_time[8];
	Sint32 drive;
	Sint32 fentry_ofs;
	Sint32 write_block;
	Sint32 rest_block;
	Sint32 start_fat;
	Sint32 prev_fat;
	Sint32 next_fat;
	Sint32 current_fat;
} BUS_FILE;

typedef BUS_FILE* BUFILE;


/******** obNAbvt@Cwb_ ***************************************/
typedef struct {
	char vms_comment[20];       /* VMSpRg                           */
	char btr_comment[36];       /* BOOT ROMpRg                      */
	char game_name[20];         /* Q[                                */
	void* icon_data;            /* ACRf[^AhX                  */
	Uint16 icon_num;            /* ACR                              */
	Uint16 icon_speed;          /* ACRAj[VXs[h          */
	void* visual_data;          /* rWAf[^AhX                */
	Uint16 visual_type;         /* rWA^Cv                        */
	Uint16 reserved;            /* \                                    */
	void* save_data;            /* AvP[VZ[uf[^AhX    */
	Uint32 save_size;           /* AvP[VZ[uf[^TCY      */
} BUS_BACKUPFILEHEADER;

/******** {[ACRt@C ***************************************/
typedef struct {
	char vms_comment[16];              /* VMSpRg                    */
	Uint8 mono_icon[128];              /* mNACRf[^           */
	Uint8 color_icon[544];             /* J[ACRf[^             */
	Uint8 reserved[1024-16-128-544-4]; /* \                             */
	Uint32 aligndata;                  /* 4oCgAĈ߂̃f[^    */
} BUS_VOLUMEICONFILE;


typedef void (*BU_INITCALLBACK)(void);
typedef Sint32 (*BU_COMPLETECALLBACK)(Sint32 drive, Sint32 op,
										Sint32 status, Uint32 param);
typedef Sint32 (*BU_PROGRESSCALLBACK)(Sint32 drive, Sint32 op,
										Sint32 count, Sint32 max);


Sint32 buInit(Sint32 capacity, Sint32 drives, void* work,
													BU_INITCALLBACK func);
Sint32 buExit(void);
void buExecServer(void);
void buSetCompleteCallback(BU_COMPLETECALLBACK func);
void buSetProgressCallback(BU_PROGRESSCALLBACK func);
void buSetCallbackParam(Uint32 param);
void buGetVersion(Sint32* major, Sint32* minor);
Sint32 buIsReady(void);
Sint32 buMount(Sint32 drive);
Sint32 buRemount(Sint32 drive);
Sint32 buUnmount(Sint32 drive);
Sint32 buIsFormat(Sint32 drive);
Sint32 buFormatDisk(Sint32 drive, const char* volume, Sint32 icon,
													const BUS_TIME* time);
Sint32 buGetDiskInfo(Sint32 drive, BUS_DISKINFO* info);
Sint32 buGetDiskFree(Sint32 drive, Sint32 type);
Sint32 buFindFirstFile(Sint32 drive, char* fname);
Sint32 buFindNextFile(Sint32 drive, char* fname);
Sint32 buFindExecFile(Sint32 drive, char* fname);
Sint32 buOpenFile(Sint32 drive, const char* fname, BUFILE* file);
Sint32 buCreateFile(Sint32 drive, const char* fname, const BUS_TIME* time,
															BUFILE* file);
Sint32 buSetFileParam(BUFILE file, Uint16 hdrofs, Uint8 copyflag);
Sint32 buCloseFile(BUFILE file);
Sint32 buReadFile(BUFILE file, void* buf, Sint32 nblock);
Sint32 buWriteFile(BUFILE file, const void* buf, Sint32 nblock);
Sint32 buDeleteFile(Sint32 drive, const char* fname);
Sint32 buSetFileAttr(Sint32 drive, const char* fname, Uint16 hdrofs,
															Uint8 copyflag);
Sint32 buIsExistFile(Sint32 drive, const char* fname);
Sint32 buGetFileInfo(Sint32 drive, const char* fname, BUS_FILEINFO* finfo);
Sint32 buStat(Sint32 drive);
Sint32 buGetFileSize(Sint32 drive, const char* fname);
Sint32 buRenameFile(Sint32 drive, const char* from, const char* to);
Sint32 buCreateExecFile(Sint32 drive, const char* fname, const BUS_TIME* time,
																BUFILE* file);
Sint32 buDefragDisk(Sint32 drive, void* work);
Sint32 buCloseFile(BUFILE file);
void buSetPrintFunc(void (*func)(const char* str));
void buSetRetryCount(Sint32 count);



Sint32 buSaveFile(Sint32 drive, const char* fname, const void* buf,
																Uint32 nblock);
Sint32 buLoadFile(Sint32 drive, const char* fname, void* buf);

Uint32 buMakeBackupFileImage(void* buf, const BUS_BACKUPFILEHEADER* hdr);
Uint32 buCalcBackupFileSize(Uint32 inum, Uint32 vtype, Uint32 size);
Sint32 buAnalyzeBackupFileImage(BUS_BACKUPFILEHEADER* hdr, const void* buf);
Uint16 buCalcCRC(const void* buf, Uint32 size);


/******** [Ne *******************************************************/
#define BUD_WORK_GAP 512

#define BUM_WORK_SIZE(_cap_, _max_) \
	(((_cap_ * _max_) * 512) + (BUD_WORK_GAP * _max_))


/******** Ήőe *****************************************************/
#define BUD_CAPACITY_128KB   (1 + 1 + 13)   /* e128KB̂               */
#define BUD_CAPACITY_256KB   (1 + 2 + 25)   /* e256KB܂               */
#define BUD_CAPACITY_512KB   (1 + 4 + 50)   /* e512KB܂               */
#define BUD_CAPACITY_1MB     (1 + 8 + 100)  /* e1MB܂                 */

/******** ΉhCu *****************************************************/
#define BUD_USE_DRIVE_A1    (1 << 0)        /* |[gÅgfoCX1      */
#define BUD_USE_DRIVE_A2    (1 << 1)        /* |[gÅgfoCX2      */
#define BUD_USE_DRIVE_B1    (1 << 2)        /* |[gB̊gfoCX1      */
#define BUD_USE_DRIVE_B2    (1 << 3)        /* |[gB̊gfoCX2      */
#define BUD_USE_DRIVE_C1    (1 << 4)        /* |[gC̊gfoCX1      */
#define BUD_USE_DRIVE_C2    (1 << 5)        /* |[gC̊gfoCX2      */
#define BUD_USE_DRIVE_D1    (1 << 6)        /* |[gD̊gfoCX1      */
#define BUD_USE_DRIVE_D2    (1 << 7)        /* |[gD̊gfoCX2      */
#define BUD_USE_DRIVE_ALL   0xff            /* S                        */

/******** hCuԍ *****************************************************/
#define BUD_DRIVE_A1   0                    /* |[gÅgfoCX1      */
#define BUD_DRIVE_A2   1                    /* |[gÅgfoCX2      */
#define BUD_DRIVE_B1   2                    /* |[gB̊gfoCX1      */
#define BUD_DRIVE_B2   3                    /* |[gB̊gfoCX2      */
#define BUD_DRIVE_C1   4                    /* |[gC̊gfoCX1      */
#define BUD_DRIVE_C2   5                    /* |[gC̊gfoCX2      */
#define BUD_DRIVE_D1   6                    /* |[gD̊gfoCX1      */
#define BUD_DRIVE_D2   7                    /* |[gD̊gfoCX2      */

/******** t@C^Cv ***************************************************/
#define BUD_FILETYPE_NORMAL       0         /* ʏt@C                */
#define BUD_FILETYPE_EXECUTABLE   1         /* st@C                */


/******** Xe[^X *******************************************************/
#define BUD_STAT_READY   0x00000000         /* R}hs\            */
#define BUD_STAT_BUSY    0xffffffff         /* R}hs              */

/******** R[obNIy[Vp[^ *****************************/
#define BUD_OP_NOP           0x00000000     /* nop                         */
#define BUD_OP_MOUNT         0x00000001     /* buRemount()/ڑ            */
#define BUD_OP_FORMATDISK    0x00000002     /* buFormatDisk()              */
#define BUD_OP_CLOSEFILE     0x00000003     /* buCloseFile()               */
#define BUD_OP_READFILE      0x00000004     /* buReadFile()                */
#define BUD_OP_WRITEFILE     0x00000005     /* buWriteFile()               */
#define BUD_OP_DELETEFILE    0x00000006     /* buDeleteFile()              */
#define BUD_OP_RENAMEFILE    0x00000007     /* buRenameFile()              */
#define BUD_OP_DEFRAGDISK    0x00000008     /* buDefragDisk()              */
#define BUD_OP_SETFILEATTR   0x00000009     /* buSetFileAttr()             */

/******** R[obNIy[Vp[^ *****************************/
#define BUD_CBRET_OK   0x00000000           /* p                    */

/******** G[R[h *****************************************************/
#define BUD_ERR_OK              0x00000000  /* G[Ȃ                  */
#define BUD_ERR_BUSY            0xffffffff  /* R}hs              */
#define BUD_ERR_INVALID_PARAM   0xffffff00  /* ֐p[^ُ        */
#define BUD_ERR_ILLEGAL_DISK    0xffffff81  /* ŏIubNs            */
#define BUD_ERR_ILLEGAL_FORMAT  0xffffff82  /* ŏIubNs            */
#define BUD_ERR_UNKNOWN_DISK    0xffffff83  /* ΉĂȂfBXN      */
#define BUD_ERR_NO_DISK         0xffffff01  /* fBXNĂȂ      */
#define BUD_ERR_UNMOUNT         0xffffff02  /* fBXNĂȂ      */
#define BUD_ERR_UNFORMAT        0xffffff03  /* tH[}bg              */
#define BUD_ERR_DISK_FULL       0xffffff04  /* fBXNt                */
#define BUD_ERR_FILE_NOT_FOUND  0xffffff05  /* t@CȂ      */
#define BUD_ERR_FILE_EXIST      0xffffff06  /* Õt@C    */
#define BUD_ERR_CANNOT_OPEN     0xffffff07  /* t@CJȂ          */
#define BUD_ERR_CANNOT_CREATE   0xffffff08  /* st@C쐬łȂ  */
#define BUD_ERR_ACCESS_DENIED   0xffffff0b  /* t@CANZX        */
#define BUD_ERR_WRITE_ERROR     0xffffff40  /* CgG[                */
#define BUD_ERR_GENERIC         0xffff0000  /* `G[                */

               /* [hI[vĂt@Cɑ΂ďƂ */
               /* NGCgI[vĂt@CǂݍƂ */
               /* I[vĂt@Cɑ΂ă[hACgA       */
               /* N[YȊȎs悤Ƃ                     */

/******** obNAbvt@C̃rWAf[^^Cv *********************/
#define BUD_VISUALTYPE_NONE   0             /* rWAf[^Ȃ        */
#define BUD_VISUALTYPE_A      1             /* ^CvA                     */
#define BUD_VISUALTYPE_B      2             /* ^CvB                     */
#define BUD_VISUALTYPE_C      3             /* ^CvC                     */


#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif /* _SG_BUP_H_ */

